"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const Jenkins = require("jenkins");
const cache_1 = require("../cache");
const utils_1 = require("../utils");
const plugin_1 = require("./plugin");
var ParameterType;
(function (ParameterType) {
    ParameterType["git"] = "git";
    ParameterType["checkbox"] = "checkbox";
    ParameterType["radio"] = "radio";
})(ParameterType = exports.ParameterType || (exports.ParameterType = {}));
class JenkinsCli {
    constructor(options) {
        this.jenkins = Jenkins(Object.assign(Object.assign({}, options.config), { promisify: true }));
        this.cacheJobs = cache_1.jobCache.getJob();
    }
    // get the job list
    getJobs() {
        return __awaiter(this, void 0, void 0, function* () {
            const info = yield this.jenkins.info();
            return info.jobs.map(job => job.name);
        });
    }
    // get job configuration information
    getConfig(selectJob) {
        return __awaiter(this, void 0, void 0, function* () {
            const configXML = yield this.jenkins.job.config(selectJob);
            return yield utils_1.loadXML(configXML);
        });
    }
    // get all parameters for the job
    getParameters(selectJob) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const config = yield this.getConfig(selectJob);
                let parameters = [];
                const parameterDefinitions = config.project.properties[0]['hudson.model.ParametersDefinitionProperty'][0].parameterDefinitions;
                parameterDefinitions.forEach(p => {
                    Object.keys(p).forEach(o => {
                        const po = p[o];
                        switch (o) {
                            case "net.uaznia.lukanus.hudson.plugins.gitparameter.GitParameterDefinition":
                                const values = plugin_1.getParametersByGit(config.project);
                                parameters.push({
                                    type: ParameterType.git,
                                    key: po[0].name[0],
                                    description: po[0].description[0],
                                    value: values
                                });
                                break;
                            case "hudson.model.ChoiceParameterDefinition":
                                parameters = parameters.concat(plugin_1.getParametersByRadio(po));
                                break;
                            case "com.cwctravel.hudson.plugins.extended__choice__parameter.ExtendedChoiceParameterDefinition":
                                parameters = parameters.concat(plugin_1.getParametersByCheckbox(po));
                                break;
                            default:
                                break;
                        }
                    });
                });
                // If it is git, get the branch from the remote
                for (const p of parameters) {
                    if (p.type === ParameterType.git) {
                        p.value = yield utils_1.getBranchByRemote(p.value);
                    }
                }
                return parameters;
            }
            catch (err) {
                utils_1.printError(err);
                return [];
            }
        });
    }
    // todo: Unable to get the latest job queue when a single job takes a lot of time
    build(selectJob, parameters) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.jenkins.job.build({ name: selectJob, parameters });
            cache_1.jobCache.refreshJob({
                name: selectJob,
                parameters
            });
            return yield this.jenkins.job.get(selectJob);
            // const log = await this.jenkins.build.logStream(selectJob, info.lastBuild.number)
            // return new Promise((resolve, reject) => {
            //     log.on('data', function(text) {
            //         process.stdout.write(text)
            //     })
            //     log.on('error', function(err) {
            //         printError(err)
            //         reject(err)
            //     })
            //     log.on('end', function() {
            //         resolve(true)
            //     })
            // })
        });
    }
}
exports.JenkinsCli = JenkinsCli;
//# sourceMappingURL=index.js.map